import json
import re
from collections import defaultdict

import pandas as pd
import pronto as pt

from ipc.validator.settings import ROOT_DIR

search_for_disease, search_for_organism = 'MESH', 'NCBITaxon'
print('Retrieving ontological names for:')

print('Tissues...')
tissues = pt.Ontology('https://raw.githubusercontent.com/bioinfo-el/ontologies/main/obo/tissue.obo')

print('Cell Types...')
cell_types = pt.Ontology('https://raw.githubusercontent.com/bioinfo-el/ontologies/main/obo/cell_type.obo')

print('Cell Lines...')
cell_line = pt.Ontology('https://raw.githubusercontent.com/bioinfo-el/ontologies/main/obo/cell_line.obo')

print('Drugs...')
drugs = pt.Ontology('https://raw.githubusercontent.com/bioinfo-el/ontologies/main/obo/drug.obo')

print('Diseases...')
diseases = pt.Ontology('https://raw.githubusercontent.com/bioinfo-el/ontologies/main/obo/disease.obo')

print('Organisms...')
organism_df = pd.read_csv('https://github.com/bioinfo-el/ontologies/blob/main/compressed/organism.xz?raw=true',
                          sep='|',
                          compression='xz')


class SetEncoder(json.JSONEncoder):
    """
    Encoder class to convert set to list when writing to file from a JSON object.
    """

    def default(self, obj):
        if isinstance(obj, set):
            return list(obj)
        return json.JSONEncoder.default(self, obj)


def collect_valid_names():
    all_names = {'tissue': set(),
                 'kw_cell_type': set(),
                 'kw_cell_line': dict(),
                 'disease': set(),
                 'kw_drug': set(),
                 'organism': set()}
    for term in tissues.terms():
        """Specific check for tissue"""
        if 'tissue' in term.subsets:
            all_names['tissue'].add(term.name)

    for term in cell_types.terms():
        all_names['kw_cell_type'].add(term.name)
    for term in diseases.terms():
        all_names['disease'].add(term.name)
    for term in drugs.terms():
        all_names['kw_drug'].add(term.name)

    all_names['organism'] = set(organism_df['root'])

    print('Creating mapping between Cell Lines and Diseases...')
    cell_line_disease_dict = defaultdict(set)
    for term in cell_line.terms():
        search_res = [item.description for item in term.xrefs if re.search(search_for_disease, item.id)]
        if search_res:
            cell_line_disease_dict[search_res[0]].add(term.name)

    print('Creating mapping between Cell Lines and Organisms...')
    cell_line_organism_dict = defaultdict(set)
    for term in cell_line.terms():
        search_res = [item.description for item in term.xrefs if re.search(search_for_organism, item.id)]
        if search_res:
            cell_line_organism_dict[search_res[0]].add(term.name)

    valid_cell_line_names = []
    for term in cell_line.terms():
        valid_cell_line_names.append(term.name)
    all_names['kw_cell_line'] = {'valid_cell_line_names': valid_cell_line_names,
                                 'organism': cell_line_organism_dict,
                                 'disease': cell_line_disease_dict}

    print('Setting predefined values...')
    """Predefined hardcoded values:"""
    all_names["kw_data_type"] = ["Transcriptomics",
                                 "Metabolomics",
                                 "Mutation",
                                 "MiRNA",
                                 "Proteomics",
                                 "Single cell",
                                 "Gene dependency",
                                 "Methylation",
                                 "Drug Screens",
                                 "Gene effect",
                                 "RNAi",
                                 "Copy Number Variation",
                                 "Cytometry",
                                 "Lab measurement",
                                 "PCR",
                                 "Titer",
                                 "Fusion"]

    all_names['dataset_source'] = ["GEO",
                                   "cBioportal",
                                   "TCGA",
                                   "Metabolomics Workbench",
                                   "DepMap",
                                   "PharmacoDB",
                                   "GDC",
                                   "CPTAC",
                                   "CCLE",
                                   "LINCS",
                                   "PPMI",
                                   "HPA",
                                   "MetaboLights",
                                   "GTEx",
                                   "ArrayExpress",
                                   "SingleCellPortal",
                                   "Figshare",
                                   "HCA",
                                   "Immport",
                                   "Zenodo",
                                   "Publication",
                                   "gnomAD",
                                   "RCSB",
                                   "SynapseAuron",
                                   "Cygnal",
                                   "NGJ",
                                   "RubiusTx",
                                   "Sigilon",
                                   "Yale",
                                   "Valo"]

    with open(ROOT_DIR / 'validator' / 'valid_names.json', 'w') as openfile:
        json.dump(all_names, openfile, cls=SetEncoder, indent=4)
